<?php

namespace App\Controller;

use App\Entity\Counter;
use App\Repository\CounterRepository;
use App\Service\CounterManager;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Workflow\WorkflowInterface;

class CounterController extends AbstractController
{
    public function __construct(
        protected CounterManager $counterManager,
        protected WorkflowInterface $countdownStateMachine,
        protected CounterRepository $counterRepository
    )
    {
        
    }

    #[Route('/counter', name: 'app_counter', methods: ['GET'])]
    public function index(CounterRepository $counterRepository): JsonResponse
    {
        $counters = $counterRepository->findAll();
        return $this->json($counters, 200);
    }

    #[Route('/counter', name: 'app_counter_post', methods: ['POST'])]
    public function postCounter(
        Request $request,
        CounterManager $counterManager,
    ): JsonResponse
    {
        $ttl = $request->toArray()['ttl'] ?? null;
        $name = $request->toArray()['name'] ?? null;

        $counter = $counterManager->init($ttl, $name);

        return $this->json($counter, 200);
    }

    #[Route('/counter/start/{id}', name: 'app_counter_start')]
    public function startCounter(Counter $counter): JsonResponse
    {
        $this->countdownStateMachine->apply($counter, Counter::TRANSITION_TO_STARTED);

        return $this->json($counter, 200);
    }

    #[Route('counter/{id}', name: 'app_counter_id', methods: ['GET'])]
    public function getCounter(Counter $counter): JsonResponse
    {
        return $this->json($counter, 200);
    }

    #[Route('counter/clear/{id}', name: 'app_counter_clear')]
    public function clear(Counter $counter): JsonResponse
    {
        $this->countdownStateMachine->apply($counter, Counter::TRANSITION_TO_READY);

        return $this->json($counter, 200);
    }
}
